﻿// Copyright (c) Dundas Data Visualization Inc. All rights reserved.
//
// THIS SOURCE CODE IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT WITHOUT ANY 
// WARRANTY. IT IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED 
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE 
// QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE 
// DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
//
// IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW THE AUTHOR WILL BE LIABLE TO YOU 
// FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES 
// ARISING OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT 
// LIMITED TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY 
// YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER 
// PROGRAMS), EVEN IF THE AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH 
// DAMAGES.

using Dundas.BI.Data.Functions;
using Dundas.BI.Data.Parameters;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CustomFunction
{
    /// <summary>
    /// This class represents a simple NetPrice function.
    /// </summary>
    /// <seealso cref="Dundas.BI.Data.Functions.FunctionDefinition" />
    public class CustomFunction : FunctionDefinition
    {
        /// <summary>
        /// Gets the alignment option for the current function.
        /// </summary>
        public override AlignmentAxis AlignmentCategory
        {
            get { return AlignmentAxis.Hierarchy; }
        }

        /// <summary>
        /// Gets the function category ID.
        /// </summary>
        public override Category CategoryId
        {
            get { return Category.Standard; }
        }

        /// <summary>
        /// Executes the function and calculates the results.
        /// </summary>
        /// <param name="dataInputs">The data input values.</param>
        /// <param name="settingValues">The function setting values.</param>
        /// <returns>
        /// The collection of function results.
        /// </returns>
        public override IList<FunctionResult> Execute(IEnumerable<FunctionInput> dataInputs, IEnumerable<Dundas.BI.Data.Parameters.ParameterValue> settingValues)
        {


            double[] listPrice = dataInputs.First(p => p.InputId == this.DataInputs[0].Id).Values;
            double[] discountPercentage = dataInputs.First(p => p.InputId == this.DataInputs[1].Id).Values;

            Collection<FunctionResult> results = new Collection<FunctionResult>();
            
            double[] result = new double[listPrice.Length];

            for (int index = 0; index < result.Length; index++)
            { 
                result[index] = 
                    listPrice[index] * ((100.0 - discountPercentage[index]) / 100);
            }

            ArrayResult arrayResult = new ArrayResult(new Guid("14460384-a8fa-494b-abf7-e9e67c6b8e11"), result);

            results.Add(arrayResult);

            return results;
        }

        /// <summary>
        /// Populates the function metadata by describing the data inputs, the function parameters and results.
        /// </summary>
        /// <param name="dataInputs">The data inputs.</param>
        /// <param name="settings">The function settings.</param>
        /// <param name="results">The results.</param>
        /// <returns>
        /// The formula symbol to be used in scripts.
        /// </returns>
        protected override string GetMetadata(
            out IList<InputDescriptor> dataInputs, 
            out IList<Dundas.BI.Data.Parameters.ComponentSetting> 
            settings, out IList<ResultDescriptor> results)
        {
            dataInputs =
                new Collection<InputDescriptor>()
                {
                    new InputDescriptor(
                        new Guid("1c77c9a2-070e-447b-8f11-9c379fa129b1"), 
                        "List Price", 
                        "The displayed price of the item.",
                        FunctionInputType.Standard
                    ),
                    new InputDescriptor(
                        new Guid("b5d7aa53-a790-4fa7-9590-9695f3615903"), 
                        "Discount Percentage", 
                        "The discount percentage.",
                        FunctionInputType.Standard),
                };

            settings = new Collection<ComponentSetting>();

            results =
                new Collection<ResultDescriptor>()
                {
                    new ResultDescriptor(
                        new Guid("3fe65b2e-4758-4452-9283-b04ecf9631a3"), 
                        this.ComponentName, 
                        this.ComponentDescription, 
                        FunctionResultType.Standard, true, 
                        string.Empty
                        )
                };


            return "NETPRICE";

        }

        /// <summary>
        /// Gets the standard component description.
        /// </summary>
        public override string ComponentDescription
        {
            get { return "This is a NETPRICE function designed as sample for a samples.dundas.com."; }
        }

        /// <summary>
        /// Gets the component ID.
        /// </summary>
        public override Guid ComponentId
        {
            get { return new Guid("1c77c9a2-070e-447b-8f11-9c379fa129b1"); }
        }

        /// <summary>
        /// Gets the standard component name.
        /// </summary>
        public override string ComponentName
        {
            get { return "NetPrice"; }
        }
    }
}
